<?php

declare(strict_types=1);

namespace Drupal\opigno_learning_path;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\ggroup\GroupHierarchyManagerInterface;
use Drupal\grequest\Plugin\GroupContentEnabler\GroupMembershipRequest;
use Drupal\group\Entity\GroupContentInterface;
use Drupal\group\Plugin\GroupContentEnabler\GroupMembership;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Centralized entity hooks logic.
 */
final class MembershipHooks implements ContainerInjectionInterface {

  use StringTranslationTrait;

  /**
   * As therein.
   */
  private GroupHierarchyManagerInterface $groupHierarchyManager;

  /**
   * The constructor.
   */
  public function __construct(
    GroupHierarchyManagerInterface $groupHierarchyManager
  ) {
    $this->groupHierarchyManager = $groupHierarchyManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('ggroup.group_hierarchy_manager')
    );
  }

  /**
   * Act on entity presave.
   */
  public function preSave(EntityInterface $entity): void {
    if (!$entity instanceof GroupContentInterface) {
      return;
    }

    $content_plugin = $entity->getContentPlugin();
    $group_type_id = $content_plugin->getGroupTypeId();

    // Act on learning path membership requests.
    if (
      $group_type_id === 'learning_path' &&
      $content_plugin instanceof GroupMembershipRequest
    ) {
      $this->convertLearningPathMembershipRequest($entity);
    }

    // Act on class memberships.
    elseif (
      $group_type_id === 'opigno_class' &&
      $content_plugin instanceof GroupMembership
    ) {
      $this->sendNewMembershipNotification($entity);
    }
  }

  /**
   * Convert membership requests to point to classes instead.
   */
  private function convertLearningPathMembershipRequest(GroupContentInterface $group_content) {
    $subgroups = $this->groupHierarchyManager->getGroupSubgroups($group_content->getGroup()->id());

    // If there are multiple classes, subscribe to the first one.
    foreach ($subgroups as $subgroup) {
      if ($subgroup->bundle() === 'opigno_class') {
        $group_content->set('gid', $subgroup->id());
        break;
      }
    }
  }

  /**
   * Send notification when joining a class.
   */
  private function sendNewMembershipNotification(GroupContentInterface $group_content) {
    if (!$group_content->isNew() || $group_content->isSyncing()) {
      return;
    }

    $uid = $group_content->getEntity()->id();
    $group = $group_content->getGroup();
    $parent_groups = $this->groupHierarchyManager->getGroupSupergroups($group->id());
    foreach ($parent_groups as $parent_group) {
      if ($parent_group->bundle() !== 'learning_path') {
        continue;
      }

      // Add notification.
      $message = $this->t('Enrolled to a new training "@name"', [
        '@name' => $parent_group->label(),
      ]);
      $url = $parent_group->toUrl()->toString();
      opigno_set_message($uid, $message, $url);
    }
  }

}
